@@ -1,18 +1,17 @@
 from __future__ import annotations
 
 import logging
-from re import I
-from typing import Any, Callable, Optional
+from typing import Any, Callable, Optional, Union
 
 import ckan.plugins.toolkit as tk
 import ckan.lib.plugins as lib_plugins
 import ckan.lib.navl.dictization_functions as df
-from ckan.logic import validate
+from ckan.logic import validate, ValidationError
 
 from ckanext.transmute.types import TransmuteData, Field
 from ckanext.transmute.schema import SchemaParser, SchemaField
 from ckanext.transmute.schema import transmute_schema, validate_schema
-from ckanext.transmute.exception import ValidationError, TransmutatorError
+from ckanext.transmute.exception import TransmutatorError
 from ckanext.transmute.utils import get_transmutator
 
 
@@ -57,7 +56,7 @@ def _transmute_data(data, definition, root="Dataset"):
 
     if not schema:
         return
-    
+
     mutate_old_fields(data, definition, root)
     create_new_fields(data, definition, root)
 
@@ -65,13 +64,13 @@ def _transmute_data(data, definition, root="Dataset"):
 def mutate_old_fields(data, definition, root):
     """Checks all of the data fields and mutate them
     according to the provided schema
-    
+
     New fields won't be created here, because we are
     only traversing the data dictionary
-    
+
     We can't traverse only Data or only Schema, because
     otherwise, the user will have to define all of the fields
-    that could exist in data 
+    that could exist in data
 
     Args:
         data (dict: [str, Any]): a data to mutate
@@ -79,7 +78,7 @@ def mutate_old_fields(data, definition, root):
         root (str): a root schema type
     """
     schema = definition.types[root]
-    
+
     for field_name, value in data.copy().items():
         field: SchemaField = schema["fields"].get(field_name)
 
@@ -91,66 +90,70 @@ def mutate_old_fields(data, definition, root):
             continue
 
         if field.default and not value:
-            data[field_name] = value = field.default
+            data[field.name] = value = field.default
 
         if field.default_from and not value:
-            data[field_name] = value = data[field.get_default_from()]
+            data[field.name] = value = _default_from(data, field)
 
         if field.replace_from:
-            data[field_name] = value = data[field.get_replace_from()]
+            data[field.name] = value = _replace_from(data, field)
 
         if field.value:
             if field.update:
-                if not isinstance(data[field_name], type(field.value)):
-                    raise ValidationError({f"{field_name}: the origin value has different type"})
+                if not isinstance(data[field.name], type(field.value)):
+                    raise ValidationError(
+                        {f"{field.name}: the origin value has different type"}
+                    )
 
-                if isinstance(data[field_name], dict):
-                    data[field_name].update(field.value)
-                elif isinstance(data[field_name], list):
-                    data[field_name].extend(field.value)
+                if isinstance(data[field.name], dict):
+                    data[field.name].update(field.value)
+                elif isinstance(data[field.name], list):
+                    data[field.name].extend(field.value)
                 else:
-                    raise ValidationError({f"{field_name}: the field value is immutable"})
+                    raise ValidationError(
+                        {f"{field.name}: the field value is immutable"}
+                    )
             else:
-                data[field_name] = value = field.value
+                data[field.name] = value = field.value
 
         if field.is_multiple():
             for nested_field in value:
                 _transmute_data(nested_field, definition, field.type)
         else:
-            data[field_name] = _apply_validators(
-                Field(field_name, value, root), field.validators
+            data[field.name] = _apply_validators(
+                Field(field.name, value, root), field.validators
             )
 
         if field.map_to:
-            data[field.map_to] = data.pop(field_name)
+            data[field.map_to] = data.pop(field.name)
 
 
 def create_new_fields(data, definition, root):
     """New fields are going to be created according
     to the provided schema
-    
+
     If the defined field is not exist in the data dict
     we are going to create it
-    
+
     The newly created field's value could be inherited from
     an existing field. This field must be defined in the
     schema.
     """
     schema = definition.types[root]
-    
+
     for field_name, field in schema["fields"].items():
         if field_name in data:
             continue
-        
-        if field.default_from:
-            data[field_name] = data[field.get_default_from()]
-            
-        if field.replace_from:
-            data[field_name] = data[field.get_replace_from()]
 
-        if field.value:
-            data[field_name] = field.value
-        
+        if field.default or field.value:
+            data[field_name] = field.default or field.value
+
+        if field.default_from:
+            data[field_name] = _default_from(data, field)
+
+        if field.replace_from:
+            data[field_name] = _replace_from(data, field)
+
         if field_name not in data:
             continue
 
@@ -159,6 +162,37 @@ def create_new_fields(data, definition, root):
         )
 
 
+def _default_from(data, field):
+    default_from: Union[list[str], str] = field.get_default_from()
+    return _get_external_fields(data, default_from)
+
+
+def _replace_from(data, field):
+    replace_from: Union[list[str], str] = field.get_replace_from()
+    return _get_external_fields(data, replace_from)
+
+
+def _get_external_fields(data, external_fields):
+    if isinstance(external_fields, list):
+        return _combine_from_fields(data, external_fields)
+    return data[external_fields]
+
+
+def _combine_from_fields(data, combine_fields: list[str]):
+    value = []
+
+    for field_name in combine_fields:
+        field_value = data[field_name]
+
+        if isinstance(field_value, list):
+            for item in data[field_name]:
+                value.append(item)
+        else:
+            value.append(field_value)
+
+    return value
+
+
 def _apply_validators(field: Field, validators: list[Callable[[Field], Any]]):
     """Applies validators sequentially to the field value
 
@@ -185,6 +219,8 @@ def _apply_validators(field: Field, validators: list[Callable[[Field], Any]]):
                 field = get_transmutator(validator)(field)
     except df.Invalid as e:
         raise ValidationError({f"{field.type}:{field.field_name}": [e.error]})
+    except TypeError as e:
+        raise TransmutatorError(str(e))
 
     return field.value
 
@@ -201,12 +237,13 @@ def validate(ctx, data_dict: dict[str, Any]) -> Optional[dict[str, str]]:
     package_plugin = lib_plugins.lookup_package_plugin(data["type"])
 
     data, errors = lib_plugins.plugin_validate(
-        package_plugin, ctx, data, schema, 'package_create')
+        package_plugin, ctx, data, schema, "package_create"
+    )
 
     return data, errors
 
 
-def _set_package_type(data_dict: dict[str, Any]) -> str:
+def _set_package_type(data_dict: dict[str, Any]):
     """Set a package type
 
     Args:
@@ -215,7 +252,7 @@ def _set_package_type(data_dict: dict[str, Any]) -> str:
     Returns:
         str: package type
     """
-    if 'type' in data_dict and data_dict['type']:
+    if "type" in data_dict and data_dict["type"]:
         return
 
     package_plugin = lib_plugins.lookup_package_plugin()
@@ -223,16 +260,17 @@ def _set_package_type(data_dict: dict[str, Any]) -> str:
     try:
         package_type = package_plugin.package_types()[0]
     except (AttributeError, IndexError):
-        package_type = 'dataset'
-    
+        package_type = "dataset"
+
     data_dict["type"] = package_type
 
+
 def _get_package_schema(ctx):
     package_plugin = lib_plugins.lookup_package_plugin()
 
-    if 'schema' in ctx:
-        schema = ctx['schema']
+    if "schema" in ctx:
+        schema = ctx["schema"]
     else:
         schema = package_plugin.create_package_schema()
 
-    return schema
\ No newline at end of file
+    return schema
