@@ -3,5 +3,5 @@ from . import get
 
 def get_auth_functions():
     return {
-        'tsm_transmute': get.transmute,
+        "tsm_transmute": get.transmute,
     }
